#!/usr/bin/env python
# -*- coding:utf-8 -*-
import os
import signal
import time

import shutil
import subprocess
import traceback
import raps.rapscomm

def get_current_time():
    return int(time.time())

def get_link_list(c_comm):
    cmd = ["find",c_comm.CONFIG.LINK_PATH,"-mindepth","2","-type","d"]
    proc = subprocess.Popen(cmd,stdout=subprocess.PIPE)
    link_list = proc.stdout.readlines()
    proc.wait()
    return link_list

def get_expire_time(link):
    link = link.rstrip('/')
    name = os.path.basename(link)
    return int(name[32:],16)

def remove_links(c_comm):
    current_time = get_current_time()
    link_list = get_link_list(c_comm)
    for link in link_list:
        link = link.strip()
        expire_time = get_expire_time(link)
        if expire_time < current_time:
            c_comm.logout("remove expired link:" + link)
            shutil.rmtree(link)

def calc_wait_time(c_comm):
    link_list = get_link_list(c_comm)
    first_expire_time = None
    for link in link_list:
        link = link.strip()
        expire_time = get_expire_time(link)
        if first_expire_time == None:
            first_expire_time = expire_time
        else:
            if first_expire_time > expire_time:
                first_expire_time = expire_time

    wait_time = 86400
    if first_expire_time != None:
        current_time = get_current_time()
        wait_time = first_expire_time - current_time
    if wait_time < 1:
        wait_time = 1
    c_comm.logout("wait_time:" + str(wait_time))

    return wait_time 

def func(signo,frame):
    pass

signal.signal(signal.SIGUSR1,func)

c_comm = raps.rapscomm.rapscomm()
c_comm.readConfig()

while True:
    try:
        c_comm.logout("link_del.py waked")
        remove_links(c_comm)
        wait_time = calc_wait_time(c_comm)
        c_comm.logout("link_del.py sleeping...")
        time.sleep(wait_time)
    except:
        traceback.print_exc(file=open(c_comm.CONFIG.LOG,"w"))
        pass

exit()

