#!/bin/sh
[ -f /etc/beta ] && set -x

FAL_SYSINFO_PATH=/mnt/data/bin/fal_sysinfo.py
FAL_NETWORK_PATH=/mnt/data/bin/fal_network.py

GETXMLTEXT_PATH=/opt/rl3/bin/getxmltext.py
GETXMLMAC_PATH=/opt/rl3/bin/getxmlmac.py

SYSINFO_SERIES_XPATH="entrylist/entry/sysinfo/series"
NETWORK_LAM1_MACADDR_XPATH="entrylist/entry/network/netiflist/netif[@name='lan1']/mac"
UNUSABLE=128

CERTS_PATH=/mnt/data/conf/apache2/certs

__is_unusable_crt() {
	local UNUSABLE_EPOCH_SEC=1507079146 #Oct  4 01:05:46 2017 GMT
	local SERVER_CRT="$1"
	local NOT_BEFORE="$(openssl x509 -noout -text -in "${SERVER_CRT}" | grep 'Not Before:' | sed 's/.*Not Before: \(.*\)$/\1/g')"
	local NOT_BEFORE_EPOCH_SEC="$(date -d "${NOT_BEFORE}" +%s)"
	if [ ${NOT_BEFORE_EPOCH_SEC} -le ${UNUSABLE_EPOCH_SEC} ]; then
		return ${UNUSABLE}
	fi
	return 0
}
__make_ssl_root_key() {
	local ROOTCA_KEY="$1"
	local ROOTCA_KEY_WORKING="${ROOTCA_KEY}.working"
	local ROOTCA_CRT="$2"
	local ROOTCA_DER="$3"
	local ROOTCA_CSR="$4"
	local ROOTCA_EXT="$5"
	local SERVER_KEY="$6"
	local NOT_BEFORE="$(openssl x509 -noout -text -in "${ROOTCA_CRT}" | grep 'Not Before:' | sed 's/.*Not Before: \(.*\)$/\1/g')"
	local NOT_AFTER="$(openssl x509 -noout -text -in "${ROOTCA_CRT}" | grep 'Not After :' | sed 's/.*Not After : \(.*\)$/\1/g')"
	local CURRENT_SEC="$(date +%s)"
	local NOT_BEFORE_SEC="$(date -d "${NOT_BEFORE}" +%s)"
	local NOT_AFTER_SEC="$(date -d "${NOT_AFTER}" +%s)"
	[ ${NOT_BEFORE_SEC} -le ${CURRENT_SEC} -a ${CURRENT_SEC} -lt ${NOT_AFTER_SEC} ] || rm -f "${ROOTCA_KEY}"
	[ -f "${ROOTCA_KEY}" ] && return 0
	rm -f "${SERVER_KEY}"
	local COUNTRY_NAME='JP'
	local STATE_OR_PROVINCE_NAME='ISHIKAWA'
	local LOCALITY_NAME='KANAZAWA'
	local ORGANIZATION_NAME='I-O DATA DEVICE, INC.'
	local ORGANIZATIONAL_UNIT_NAME='Planning and Development Dept.'
	local PRODUCT="$(echo '<request><entrylist><entry type="query" /></entrylist></request>' | ${FAL_SYSINFO_PATH} | \
		${GETXMLTEXT_PATH} ${SYSINFO_SERIES_XPATH})"
	local MACADDR="$(echo '<request><entrylist><entry type="query"><network/></entry></entrylist></request>' | ${FAL_NETWORK_PATH} | \
		${GETXMLMAC_PATH} | sed 's/..:..:..:\(..\):\(..\):\(..\)/\1\2\3/' | tr a-z A-Z)"
	local COMMON_NAME="I-O DATA ${PRODUCT} ${MACADDR}"
	openssl genrsa -out "${ROOTCA_KEY_WORKING}" 2048
	openssl req -new -key "${ROOTCA_KEY_WORKING}" -out "${ROOTCA_CSR}" \
		-subj "/C=${COUNTRY_NAME}/ST=${STATE_OR_PROVINCE_NAME}/L=${LOCALITY_NAME}/O=${ORGANIZATION_NAME}/OU=${ORGANIZATIONAL_UNIT_NAME}/CN=${COMMON_NAME}"
	echo "[v3_ca]" > "${ROOTCA_EXT}"
	echo "subjectKeyIdentifier=hash" >> "${ROOTCA_EXT}"
	echo "authorityKeyIdentifier=keyid:always,issuer" >> "${ROOTCA_EXT}"
	echo "basicConstraints=critical,CA:true" >> "${ROOTCA_EXT}"
	openssl x509 -req -signkey "${ROOTCA_KEY_WORKING}" \
		-extfile "${ROOTCA_EXT}" -extensions "v3_ca" \
		-in "${ROOTCA_CSR}" -out "${ROOTCA_CRT}" \
		-days 3650 -sha256 || return 1
	chmod 400 "${ROOTCA_KEY_WORKING}" || return 1
	openssl x509 -inform PEM -outform DER -in "${ROOTCA_CRT}" -out "${ROOTCA_DER}" || return 1
	mv "${ROOTCA_KEY_WORKING}" "${ROOTCA_KEY}" || return 1
	return 0
}

__make_ssl_server_key() {
	local SERVER_KEY="$1"
	local SERVER_KEY_WORKING="${SERVER_KEY}.working"
	local SERVER_CRT="$2"
	local SERVER_CSR="$3"
	local SERVER_EXT="$4"
	local ROOTCA_KEY="$5"
	local ROOTCA_CRT="$6"
	local NOT_BEFORE="$(openssl x509 -noout -text -in "${SERVER_CRT}" | grep 'Not Before:' | sed 's/.*Not Before: \(.*\)$/\1/g')"
	local NOT_AFTER="$(openssl x509 -noout -text -in "${SERVER_CRT}" | grep 'Not After :' | sed 's/.*Not After : \(.*\)$/\1/g')"
	local CURRENT_SEC="$(date +%s)"
	local NOT_BEFORE_SEC="$(date -d "${NOT_BEFORE}" +%s)"
	local NOT_AFTER_SEC="$(date -d "${NOT_AFTER}" +%s)"
	[ ${NOT_BEFORE_SEC} -le ${CURRENT_SEC} -a ${CURRENT_SEC} -lt ${NOT_AFTER_SEC} ] || rm -f "${SERVER_KEY}"
	[ -f "${SERVER_KEY}" ] && return 0
	local COUNTRY_NAME='JP'
	local STATE_OR_PROVINCE_NAME='ISHIKAWA'
	local LOCALITY_NAME='KANAZAWA'
	local ORGANIZATION_NAME='I-O DATA DEVICE, INC.'
	local ORGANIZATIONAL_UNIT_NAME='Planning and Development Dept.'
	openssl genrsa -out "${SERVER_KEY_WORKING}" 2048
	openssl req -new -key "${SERVER_KEY_WORKING}" -out "${SERVER_CSR}" \
		-subj "/C=${COUNTRY_NAME}/ST=${STATE_OR_PROVINCE_NAME}/L=${LOCALITY_NAME}/O=${ORGANIZATION_NAME}/OU=${ORGANIZATIONAL_UNIT_NAME}/CN=localhost"
	echo "[v3_server]" > "${SERVER_EXT}"
	echo "basicConstraints=CA:false" >> "${SERVER_EXT}"
	echo "subjectKeyIdentifier=hash" >> "${SERVER_EXT}"
	echo "authorityKeyIdentifier=keyid,issuer" >> "${SERVER_EXT}"
	echo "extendedKeyUsage=serverAuth" >> "${SERVER_EXT}"
	echo "subjectAltName=DNS:localhost" >> "${SERVER_EXT}"
	openssl x509 -req -CAkey "${ROOTCA_KEY}" -CA "${ROOTCA_CRT}" \
		-CAcreateserial -extfile "${SERVER_EXT}" \
		-extensions "v3_server" -in "${SERVER_CSR}" \
		-out "${SERVER_CRT}" -days 825 -sha256 || return 1
	chmod 400 "${SERVER_KEY_WORKING}" || return 1
	mv "${SERVER_KEY_WORKING}" "${SERVER_KEY}" || return 1
	return 0
}

mkdir -p ${CERTS_PATH}

__is_unusable_crt server.crt
ret="$(echo $?)"
if [ ${ret} -eq ${UNUSABLE} ]; then
	rm ${CERTS_PATH}/rootca.key
	rm ${CERTS_PATH}/server.key
	rm ${CERTS_PATH}/server.der
fi

__make_ssl_root_key \
	${CERTS_PATH}/rootca.key \
	${CERTS_PATH}/rootca.crt \
	${CERTS_PATH}/rootca.der \
	${CERTS_PATH}/rootca.csr \
	${CERTS_PATH}/rootca.ext \
	${CERTS_PATH}/server.key
__make_ssl_server_key \
	${CERTS_PATH}/server.key \
	${CERTS_PATH}/server.crt \
	${CERTS_PATH}/server.csr \
	${CERTS_PATH}/server.ext \
	${CERTS_PATH}/rootca.key \
	${CERTS_PATH}/rootca.crt

