#!/usr/bin/env python

import cgi
import cgitb
import sys
import os
import uuid
import hashlib
import json
from ionas.request import (Entry, Request)
from iobase.cmdexec import (CmdExec, PIPE,)
from fal_info import (FAL_USERS_PATH, FAL_NARSUS_PATH, FAL_FEATURES_PATH,
                        FAL_REMOTELINK3_PATH,)
from product_conf import (SUDO_CMD, HOSTNAME_CMD, TEMP_DIR, SYSFS_DIR,)
from product_conf.reqinfo import (ReqTypes, Results)
from product_conf.devinfo import (NET_RAW_DEV_LIST,)

cgitb.enable()


DEVCTRL_SESSION_DIR = TEMP_DIR + "/devctrl/session"
SYS_MAC_ADDRESS_PATH_TMPL = SYSFS_DIR + "/class/net/%s/address"

REMOTELINK3_FUNCTION_XPATH = """functionlist/function[text()="remotelink3"]"""
NARSUS_FUNCTION_XPATH = """functionlist/function[text()="narsus"]"""
PERSONAL_SHARE_FUNCTION_XPATH = \
                        """functionlist/function[text()="personal_share"]"""

GET_PINCODE_REQ = "get_pincode"
PINCODE_PATH = "pincode"
NARSUS_USECODE_PATH = "usecode"


RESPONSE_BASE_DICT = {
        "response": "",
        "lang":  "ja",
        "result": 0,
    }

INVALID_RESPONSE_DICT = {
        "response": "",
        "lang":  "ja",
        "result": 1,
    }

DEVICE_INFO_DICT = {
        "api_version": "1.0",
        "product": "",
        "version": "",
        "host_name": "",
        "mac_address_list": [],
    }

FEATURES_DICT = {
        "supported": {
                "personal_share": False,
                "admin_config": True,
        }
    }


def _request(fal_cmd, entry):
    response = Request(fal_cmd, filter_cmd=SUDO_CMD).request(entry)
    return response.get_entrylist().get_entry().get()


def _update_session_time(session_id):

    os.utime(os.path.join(DEVCTRL_SESSION_DIR, session_id), None)


def check_session(session_id):

    if not session_id:
        raise Exception()

    if not os.path.exists(os.path.join(DEVCTRL_SESSION_DIR, session_id)):
        raise Exception()

    _update_session_time(session_id)


def get_info(features_config):

    device_dict = dict(DEVICE_INFO_DICT)
    with open("/var/lib/landisk/product_full") as product:
        device_dict["product"] = product.read().strip()

    with open("/var/lib/landisk/version") as version:
        device_dict["version"] = version.read().strip()

    try:
        proc = CmdExec([HOSTNAME_CMD]).execute(stdout=PIPE)
        device_dict["host_name"] = proc.stdout.read().strip()
    except:
        pass

    for net_raw_dev in NET_RAW_DEV_LIST:
        with open(SYS_MAC_ADDRESS_PATH_TMPL % (net_raw_dev,)) as mac:
            device_dict["mac_address_list"].append(mac.read().strip())

    try:
        features_config.get_node(REMOTELINK3_FUNCTION_XPATH)
        device_dict["remotelink3"] = {}
        pincode_info = _request(FAL_REMOTELINK3_PATH,
                                    Entry(GET_PINCODE_REQ, "remotelink3"))
        pincode = pincode_info.get_value(PINCODE_PATH, "")
        device_dict["remotelink3"]["pin_code"] = pincode.strip()
    except:
        # remotelink3 is not supported
        pass

    try:
        function_node = features_config.get_node(NARSUS_FUNCTION_XPATH)
    except:
        # NarSuS is not supported
        return device_dict

    try:
        narsus_config = _request(FAL_NARSUS_PATH, Entry(ReqTypes.LOAD))
        usecode = narsus_config.get_node(NARSUS_USECODE_PATH)
        device_dict["narsus"] = {}
        if usecode.is_enabled():
            device_dict["narsus"]["registered"] = True
        else:
            device_dict["narsus"]["registered"] = False
    except:
        pass

    return device_dict


def get_features(features_config):

    features_dict = dict(FEATURES_DICT)

    try:
        function_node = features_config.get_node(PERSONAL_SHARE_FUNCTION_XPATH)
        features_dict["supported"]["personal_share"] = True
    except:
        # personal_share is not supported
        pass

    return features_dict

#
# main
#

print "Content-Type: text/json\n"


try:
    request = json.loads(sys.stdin.read())
    response = dict(RESPONSE_BASE_DICT)
    response["result"] = 1

    if request["request"] == "get_info":
        features_config = _request(FAL_FEATURES_PATH, Entry(ReqTypes.LOAD))

        response["response"] = "get_info"
        response["session_id"] = request["session_id"]
        check_session(request["session_id"])
        response["device_info"] = get_info(features_config)
        response["features"] = get_features(features_config)
        response["result"] = 0

    print json.dumps(response, indent=1, ensure_ascii=False).encode("utf8")
except:
    print json.dumps(INVALID_RESPONSE_DICT,
                    indent=1, ensure_ascii=False).encode("utf8")
